@@ -1,14 +1,16 @@
 import os
-import numpy as np
 from tglc.target_lightcurve import epsf
 from tglc.ffi_cut import ffi_cut
 from astroquery.mast import Catalogs
 
 # warnings.simplefilter('ignore', UserWarning)
+from threadpoolctl import ThreadpoolController, threadpool_limits
+import numpy as np
+controller = ThreadpoolController()
 
-
+@controller.wrap(limits=1, user_api='blas')
 def tglc_lc(target='NGC 7654', local_directory='', size=90, save_aper=True, limit_mag=16, get_all_lc=False,
-            first_sector_only=False, sector=None):
+            first_sector_only=False, sector=None, prior=None):
     '''
     Generate light curve for a single target.
 
@@ -25,27 +27,27 @@ def tglc_lc(target='NGC 7654', local_directory='', size=90, save_aper=True, limi
     os.makedirs(local_directory + f'source/', exist_ok=True)
     source = ffi_cut(target=target, size=size, local_directory=local_directory, sector=sector)  # sector
     catalogdata = Catalogs.query_object(str(target), radius=0.02, catalog="TIC")
-    name = 'Gaia DR2 ' + str(np.array(catalogdata['GAIA'])[0])
+    name = 'Gaia DR3 ' + str(np.array(catalogdata['GAIA'])[0])
     if get_all_lc:
         name = None
     for j in range(len(source.sector_table)):
-#         try:
+        # try:
         source.select_sector(sector=source.sector_table['sector'][j])
         epsf(source, factor=2, sector=source.sector, target=target, local_directory=local_directory,
-                 name=name, limit_mag=limit_mag, save_aper=save_aper)
+                 name=name, limit_mag=limit_mag, save_aper=save_aper, prior=prior)
         if first_sector_only:
             break
-#         except:
-#             sector_num = source.sector_table["sector"][j]
-#             warnings.warn(f'Skipping sector {sector_num}. (Target not in cut)')
+        else:
+            sector_num = source.sector_table["sector"][j]
+            # warnings.warn(f'Skipping sector {sector_num}. (Target not in cut)')
 
 
 if __name__ == '__main__':
-    target = input('Target ID or coordinates: ')
-    local_directory = f'/home/tehan/data/cosmos/{target}/'
+    target = 'TIC 270022476'
+    local_directory = f'/home/tehan/Downloads/tglc/{target}/'
     os.makedirs(local_directory, exist_ok=True)
-    tglc_lc(target=target, local_directory=local_directory, size=50, save_aper=False, limit_mag=15,
-                    get_all_lc=True, first_sector_only=True, sector=True)
+    tglc_lc(target=target, local_directory=local_directory, size=90, save_aper=False, limit_mag=16,
+                    get_all_lc=True, first_sector_only=True, sector=None, prior=None)
 
     ####### list of targets example
     # local_directory = '/home/tehan/data/ob_associations/'
