@@ -5,19 +5,22 @@ import os
 import warnings
 import numpy as np
 import numpy.ma as ma
-
+import tglc
+import matplotlib.pyplot as plt
 from astropy.io import fits
 from tqdm import trange
 from os.path import exists
-from tglc.effective_psf import get_psf, fit_psf, fit_lc, bg_mod
+from tglc.effective_psf import get_psf, fit_psf, fit_lc, fit_lc_float_field, bg_mod
+from tglc.ffi import Source
 from tglc.ffi_cut import Source_cut
+
 warnings.simplefilter('always', UserWarning)
 
 
 def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_psf_lc=None, aper_lc=None,
               cal_aper_lc=None, bg=None, tess_flag=None, tglc_flag=None, cadence=None, aperture=None,
               cut_x=None, cut_y=None, star_x=2, star_y=2, x_aperture=None, y_aperture=None, near_edge=False,
-              local_bg=None, save_aper=False):
+              local_bg=None, save_aper=False, portion=1, prior=None):
     """
     lc output to .FITS file in MAST HLSP standards
     :param tglc_flag: np.array(), required
@@ -60,7 +63,7 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
     if np.isnan(cal_aper_err):
         cal_aper_err = 'NaN'
     try:
-        ticid = source.tic['ID'][np.where(source.tic['GAIA'] == str(objid))][0]
+        ticid = str(source.tic['TIC'][np.where(source.tic['dr3_source_id'] == objid)][0])
     except:
         ticid = ''
     try:
@@ -88,8 +91,8 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
         fits.Card('TELESCOP', 'TESS', 'telescope'),
         fits.Card('INSTRUME', 'TESS Photometer', 'detector type'),
         fits.Card('FILTER', 'TESS', 'the filter used for the observations'),
-        fits.Card('OBJECT', source.gaia[index]['designation'], 'string version of Gaia DR2 ID'),
-        fits.Card('GAIADR2', objid, 'integer version of Gaia DR2 ID'),
+        fits.Card('OBJECT', source.gaia[index]['designation'], 'string version of Gaia DR3 ID'),
+        fits.Card('GAIADR3', objid, 'integer version of Gaia DR3 ID'),
         fits.Card('TICID', ticid, 'TESS Input Catalog ID'),
         fits.Card('SECTOR', source.sector, 'observation sector'),
         fits.Card('CAMERA', source.camera, 'camera No.'),
@@ -100,10 +103,10 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
         fits.Card('RADESYS', 'ICRS', 'reference frame of celestial coordinates'),
         fits.Card('RA_OBJ', source.gaia[index]['ra'], '[deg] right ascension, J2000'),
         fits.Card('DEC_OBJ', source.gaia[index]['dec'], '[deg] declination, J2000'),
-        fits.Card('TESSMAG', source.gaia[index]['tess_mag'], 'TESS magnitude, fitted by Gaia DR2 bands'),
-        fits.Card('GAIA_G', source.gaia[index]['phot_g_mean_mag'], 'Gaia DR2 g band magnitude'),
-        fits.Card('GAIA_bp', gaia_bp, 'Gaia DR2 bp band magnitude'),
-        fits.Card('GAIA_rp', gaia_rp, 'Gaia DR2 rp band magnitude'),
+        fits.Card('TESSMAG', source.gaia[index]['tess_mag'], 'TESS magnitude, fitted by Gaia DR3 bands'),
+        fits.Card('GAIA_G', source.gaia[index]['phot_g_mean_mag'], 'Gaia DR3 g band magnitude'),
+        fits.Card('GAIA_bp', gaia_bp, 'Gaia DR3 bp band magnitude'),
+        fits.Card('GAIA_rp', gaia_rp, 'Gaia DR3 rp band magnitude'),
         fits.Card('RAWFLUX', raw_flux, 'median flux of raw FFI'),
         fits.Card('CALIB', 'TGLC', 'pipeline used for image calibration')])
     if save_aper:
@@ -123,7 +126,7 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
     c2 = fits.Column(name='psf_flux', array=np.array(psf_lc), format='E')  # psf factor
     # c3 = fits.Column(name='psf_flux_err',
     #                  array=1.4826 * np.median(np.abs(psf_lc - np.median(psf_lc))) * np.ones(len(psf_lc)), format='E')
-    c4 = fits.Column(name='aperture_flux', array=aper_lc, format='E')
+    c4 = fits.Column(name='aperture_flux', array=aper_lc / portion, format='E')
     # c5 = fits.Column(name='aperture_flux_err',
     #                  array=1.4826 * np.median(np.abs(aper_lc - np.median(aper_lc))) * np.ones(len(aper_lc)), format='E')
     c6 = fits.Column(name='cal_psf_flux', array=np.array(cal_psf_lc), format='E')
@@ -146,9 +149,9 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
     table_hdu.header.append(('TELESCOP', 'TESS', 'telescope'), end=True)
     table_hdu.header.append(('INSTRUME', 'TESS Photometer', 'detector type'), end=True)
     table_hdu.header.append(('FILTER', 'TESS', 'the filter used for the observations'), end=True)
-    table_hdu.header.append(('OBJECT', source.gaia[index]['designation'], 'string version of Gaia DR2 ID'),
+    table_hdu.header.append(('OBJECT', source.gaia[index]['designation'], 'string version of Gaia DR3 ID'),
                             end=True)
-    table_hdu.header.append(('GAIADR2', objid, 'integer version of GaiaDR2 designation'), end=True)
+    table_hdu.header.append(('GAIADR3', objid, 'integer version of GaiaDR3 designation'), end=True)
     table_hdu.header.append(('RADESYS', 'ICRS', 'reference frame of celestial coordinates'), end=True)
     table_hdu.header.append(('RA_OBJ', source.gaia[index]['ra'], '[deg] right ascension, J2000'), end=True)
     table_hdu.header.append(('DEC_OBJ', source.gaia[index]['dec'], '[deg] declination, J2000'), end=True)
@@ -172,10 +175,12 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
     table_hdu.header.append(('CPSF_ERR', cal_psf_err, '[e-/s] calibrated PSF flux error'), end=True)
     table_hdu.header.append(('CAPE_ERR', cal_aper_err, '[e-/s] calibrated aperture flux error'), end=True)
     table_hdu.header.append(('NEAREDGE', near_edge, 'distance to edges of FFI <= 2'), end=True)
-    table_hdu.header.append(('LOC_BG', local_bg, 'locally modified background'), end=True)
+    table_hdu.header.append(('LOC_BG', local_bg, '[e-/s] locally modified background'), end=True)
     table_hdu.header.append(('COMMENT', "TRUE_BG = hdul[1].data['background'] + LOC_BG"), end=True)
     table_hdu.header.append(('WOTAN_WL', 1, 'wotan detrending window length'), end=True)
     table_hdu.header.append(('WOTAN_MT', 'biweight', 'wotan detrending method'), end=True)
+    if type(prior) == float:
+        table_hdu.header.append(('PRIOR', prior, 'prior of field stars'), end=True)
 
     hdul = fits.HDUList([primary_hdu, table_hdu])
     hdul.writeto(
@@ -185,7 +190,7 @@ def lc_output(source, local_directory='', index=0, time=None, psf_lc=None, cal_p
 
 
 def epsf(source, psf_size=11, factor=2, local_directory='', target=None, cut_x=0, cut_y=0, sector=0,
-         limit_mag=16, edge_compression=1e-4, power=1.4, name=None, save_aper=False, no_progress_bar=False):
+         limit_mag=16, edge_compression=1e-4, power=1.4, name=None, save_aper=False, no_progress_bar=False, prior=None):
     """
     User function that unites all necessary steps
     :param source: TGLC.ffi_cut.Source or TGLC.ffi_cut.Source_cut, required
@@ -220,7 +225,7 @@ def epsf(source, psf_size=11, factor=2, local_directory='', target=None, cut_x=0
     os.makedirs(lc_directory, exist_ok=True)
     # sim_image = np.dot(A[:source.size ** 2, :], fit_psf(A, source, over_size, power=power, time=0).T)
     # residual = np.abs(source.flux[0].flatten() - sim_image)
-    # return np.mean(residual), np.median(residual)
+    # return residual
 
     epsf_exists = exists(epsf_loc)
     if epsf_exists:
@@ -239,24 +244,28 @@ def epsf(source, psf_size=11, factor=2, local_directory='', target=None, cut_x=0
     # np.save('/mnt/c/users/tehan/desktop/7654/contamination_8_.npy', contamination_8)
     # TODO: quality use which background?
     background = np.dot(A[:source.size ** 2, -bg_dof:], e_psf[:, -bg_dof:].T)
-    # np.save(f'{local_directory}epsf/bg_{target}_sector_{sector}.npy', background)
     quality_raw = np.zeros(len(source.time), dtype=np.int16)
     sigma = 1.4826 * np.nanmedian(np.abs(e_psf[:, -1] - np.nanmedian(e_psf[:, -1])))
     quality_raw[abs(e_psf[:, -1] - np.nanmedian(e_psf[:, -1])) >= 3 * sigma] += 1
     index_1 = np.where(np.array(source.quality) == 0)[0]
     index_2 = np.where(quality_raw == 0)[0]
     index = np.intersect1d(index_1, index_2)
-    x_left = 1.5 if cut_x != 0 else -0.5
-    x_right = 2.5 if cut_x != 13 else 0.5
-    y_left = 1.5 if cut_y != 0 else -0.5
-    y_right = 2.5 if cut_y != 13 else 0.5
+    if type(source) == Source_cut:
+        in_frame = np.where(np.invert(np.isnan(source.flux[0])))
+        x_left = np.min(in_frame[1]) - 0.5
+        x_right = source.size - np.max(in_frame[1]) + 0.5
+        y_left = np.min(in_frame[0]) - 0.5
+        y_right = source.size - np.max(in_frame[0]) + 0.5
+    else:
+        x_left = 1.5 if cut_x != 0 else -0.5
+        x_right = 2.5 if cut_x != 13 else 0.5
+        y_left = 1.5 if cut_y != 0 else -0.5
+        y_right = 2.5 if cut_y != 13 else 0.5
+
     num_stars = np.array(source.gaia['tess_mag']).searchsorted(limit_mag, 'right')
     x_aperture = source.gaia[f'sector_{source.sector}_x'] - np.maximum(0, x_round - 2)
     y_aperture = source.gaia[f'sector_{source.sector}_y'] - np.maximum(0, y_round - 2)
 
-    # mag = []
-    # mean_diff_aper = []
-    # mean_diff_psf = []
     start = 0
     end = num_stars
     if name is not None:
@@ -264,31 +273,44 @@ def epsf(source, psf_size=11, factor=2, local_directory='', target=None, cut_x=0
         end = start + 1
     for i in trange(start, end, desc='Fitting lc', disable=no_progress_bar):
         if x_left <= x_round[i] < source.size - x_right and y_left <= y_round[i] < source.size - y_right:
-            if 1.5 <= x_round[i] < source.size - 2.5 and 1.5 <= y_round[i] < source.size - 2.5:
+            if type(source) == Source:
+                x_left = 1.5
+                x_right = 2.5
+                y_left = 1.5
+                y_right = 2.5
+            if x_left + 2 <= x_round[i] < source.size - (x_right + 2) and y_left + 2 <= y_round[i] < source.size - (
+                    y_right + 2):
                 near_edge = False
             else:
                 near_edge = True
-            aperture, psf_lc, star_y, star_x, portion = fit_lc(A, source, star_info=star_info, x=x_round[i],
-                                                               y=y_round[i], star_num=i, e_psf=e_psf,
-                                                               near_edge=near_edge)
-            aper_lc = np.sum(aperture[:, max(0, star_y - 1):min(5, star_y + 2), max(0, star_x - 1):min(5, star_x + 2)],
-                             axis=(1, 2))
-            local_bg, aper_lc, psf_lc, cal_aper_lc, cal_psf_lc = bg_mod(source, q=index, portion=portion, psf_lc=psf_lc,
+
+            if type(prior) == float:
+                aperture, psf_lc, star_y, star_x, portion = \
+                    fit_lc_float_field(A, source, star_info=star_info, x=x_round, y=y_round, star_num=i, e_psf=e_psf,
+                                       near_edge=near_edge, prior=prior)
+            else:
+                aperture, psf_lc, star_y, star_x, portion = \
+                    fit_lc(A, source, star_info=star_info, x=x_round[i], y=y_round[i], star_num=i, e_psf=e_psf,
+                           near_edge=near_edge)
+
+            aper_lc = np.sum(
+                aperture[:, max(0, star_y - 1):min(5, star_y + 2), max(0, star_x - 1):min(5, star_x + 2)],
+                axis=(1, 2))
+            local_bg, aper_lc, psf_lc, cal_aper_lc, cal_psf_lc = bg_mod(source, q=index, portion=portion,
+                                                                        psf_lc=psf_lc,
                                                                         aper_lc=aper_lc,
                                                                         near_edge=near_edge, star_num=i)
-            # mag.append(source.gaia['tess_mag'][i])
-            # mean_diff_aper.append(np.nanmean(np.abs(np.diff(aper_lc[index])) / portion))
-            # mean_diff_psf.append(np.nanmean(np.abs(np.diff(psf_lc[index]))))
             background_ = background[x_round[i] + source.size * y_round[i], :]
             quality = np.zeros(len(source.time), dtype=np.int16)
             sigma = 1.4826 * np.nanmedian(np.abs(background_ - np.nanmedian(background_)))
             quality[abs(background_ - np.nanmedian(background_)) >= 5 * sigma] += 1
-            lc_output(source, local_directory=lc_directory, index=i,
-                      tess_flag=source.quality, cut_x=cut_x, cut_y=cut_y, cadence=source.cadence,
-                      aperture=aperture.astype(np.float32), star_y=y_round[i], star_x=x_round[i], tglc_flag=quality,
-                      bg=background_, time=source.time, psf_lc=psf_lc, cal_psf_lc=cal_psf_lc, aper_lc=aper_lc,
-                      cal_aper_lc=cal_aper_lc, local_bg=local_bg, x_aperture=x_aperture[i], y_aperture=y_aperture[i],
-                      near_edge=near_edge, save_aper=save_aper)
-    # np.save(local_directory + f'mean_diff_aper_{target}.npy', np.array([mag, mean_diff_aper]))
-    # np.save(local_directory + f'mean_diff_psf_{target}.npy', np.array([mag, mean_diff_psf]))
-
+            if np.isnan(aper_lc).all():
+                continue
+            else:
+                lc_output(source, local_directory=lc_directory, index=i,
+                          tess_flag=source.quality, cut_x=cut_x, cut_y=cut_y, cadence=source.cadence,
+                          aperture=aperture.astype(np.float32), star_y=y_round[i], star_x=x_round[i], tglc_flag=quality,
+                          bg=background_, time=source.time, psf_lc=psf_lc, cal_psf_lc=cal_psf_lc, aper_lc=aper_lc,
+                          cal_aper_lc=cal_aper_lc, local_bg=local_bg, x_aperture=x_aperture[i],
+                          y_aperture=y_aperture[i], near_edge=near_edge, save_aper=save_aper, portion=portion,
+                          prior=prior)
